import { LanguageModel } from 'ai';
import { z } from 'zod';

export declare type Application = {
    processMessages(messages: Message[], options?: ProcessMessageOptions): Promise<ProcessMessageResult>;
};

export declare type ApplicationConfig = {
    chatModel: ChatModel;
    systemPrompt: (context: RequestContext) => Promise<string> | string;
    plugins?: ApplicationPlugin[];
    errorHandler?: (error: unknown) => Promise<MessageResponse> | MessageResponse;
};

export declare type ApplicationPlugin = {
    name: string;
    middleware?: Middleware;
    effects?: Effect[];
    tools?: ApplicationTool[];
};

export declare type ApplicationTool<TSchema extends z.ZodSchema = z.ZodTypeAny, TOutput = unknown> = {
    name: string;
    description: string;
    parameters: TSchema;
    handler: (params: z.infer<TSchema>, context: RequestContext) => Promise<TOutput>;
};

export declare type AssistantMessage = {
    role: 'assistant';
    content: string;
    senderId?: string;
    attachments?: FileBuffer[];
};

export declare type AssistantResponse = {
    role: 'assistant';
    content: string;
    usage: ModelUsage;
};

export declare interface ChatModel {
    generateResponse(context: RequestContext): Promise<AssistantResponse>;
}

export declare type Command = {
    trigger: string | RegExp;
    handler: (context: RequestContext) => Promise<string> | string;
    isAllowed?: (context: RequestContext) => boolean;
};

export declare type CommandsPluginConfig = {
    prefix: string;
    commands: Command[];
};

export declare function createApp(config: ApplicationConfig): Application;

export declare function createCommandsPlugin({ commands, prefix, }: CommandsPluginConfig): ApplicationPlugin;

export declare type DocumentReference = {
    title: string;
    url: string;
    updatedAt?: Date;
    debugInfo?: string;
};

declare type Effect = (context: RequestContext, response: MessageResponse) => Promise<void>;

export declare type EntityInfo = Record<string, object | null>;

export declare type FileBuffer = {
    name: string;
    mimeType: string;
    buffer: ArrayBuffer;
};

export declare const getReferenceStorage: () => ReferenceStorage;

export declare const loggingPlugin: ApplicationPlugin;

export declare type Message = UserMessage | AssistantMessage;

export declare interface MessageRequestExtras {
}

export declare type MessageResponse = AssistantResponse | SystemResponse;

export declare type MessageRole = Message['role'];

declare type Middleware = (context: RequestContext, next: NextFunction) => Promise<MessageResponse>;

export declare type ModelUsage = {
    model: string;
    inputTokens: number;
    outputTokens: number;
    requests: number;
    responseTime: number;
    usedTools?: Record<string, number>;
};

declare type NextFunction = () => Promise<MessageResponse>;

declare type PerformanceEntry_2 = PerformanceMark_2 | PerformanceMeasure_2;
export { PerformanceEntry_2 as PerformanceEntry }

declare type PerformanceMark_2 = {
    entryType: 'mark';
    name: string;
    startTime: number;
    detail?: any;
};
export { PerformanceMark_2 as PerformanceMark }

export declare const PerformanceMarks: {
    readonly processMessages: "processMessages";
    readonly middlewareBeforeHandler: "middleware:beforeHandler";
    readonly middlewareAfterHandler: "middleware:afterHandler";
    readonly chatModel: "chatModel";
    readonly toolExecution: "toolExecution";
    readonly errorHandler: "errorHandler";
};

declare type PerformanceMeasure_2 = {
    entryType: 'measure';
    name: string;
    startTime: number;
    duration: number;
};
export { PerformanceMeasure_2 as PerformanceMeasure }

export declare class PerformanceTimeline {
    private marks;
    private measures;
    private marksTimes;
    readonly timingOrigin: number;
    constructor();
    mark(name: string): void;
    measure(name: string, startMark: string, endMark?: string): void;
    markStart(name: string): void;
    markEnd(name: string): void;
    getMarks(): PerformanceMark_2[];
    getMeasures(): PerformanceMeasure_2[];
    getMeasureTotal(name: string): number;
    toJSON(): Record<string, unknown>;
    private _resolveMark;
}

export declare type ProcessMessageOptions = {
    extras?: MessageRequestExtras;
    onPartialResponse?: (partialText: string) => void;
};

declare type ProcessMessageResult = {
    response: MessageResponse;
    pendingEffects: Promise<void[]>;
};

export declare type ReferenceStorage = {
    addReference(ref: DocumentReference | DocumentReference[]): void;
    getReferences(): readonly DocumentReference[];
};

export declare type RequestContext = {
    /** All messages from given conversation */
    messages: Message[];
    /** Convenience reference to the last `messages` item which is the latest `UserMessage`. */
    lastMessage: UserMessage;
    tools: ApplicationTool[];
    references: ReferenceStorage;
    resolvedEntities: EntityInfo;
    systemPrompt: () => Promise<string> | string;
    onPartialResponse?: (text: string) => void;
    extras: MessageRequestExtras;
    performance: PerformanceTimeline;
};

export declare const SUPPORTED_ATTACHMENT_TYPES: string[];

export declare type SystemResponse = {
    role: 'system';
    content: string;
    error?: unknown;
    usage?: undefined;
};

export declare class UnsupportedAttachmentsException extends Error {
    readonly attachments: FileBuffer[];
    constructor(attachments: FileBuffer[]);
}

export declare type UserMessage = {
    role: 'user';
    content: string;
    senderId?: string;
    attachments?: FileBuffer[];
};

export declare class VercelChatModelAdapter implements ChatModel {
    private readonly _options;
    constructor(_options: VercelChatModelAdapterOptions);
    generateResponse(context: RequestContext): Promise<AssistantResponse>;
    private executeRequestWithStreaming;
    private executeRequest;
    private getUsageFromExecutionResult;
}

export declare type VercelChatModelAdapterOptions = {
    languageModel: LanguageModel;
    maxTokens?: number;
    maxSteps?: number;
};

export { }
